@@ -18,22 +18,14 @@
  *
  */
 
-const Mainloop = imports.mainloop;
 const Signals = imports.signals;
 
-const Atk = imports.gi.Atk;
-const Clutter = imports.gi.Clutter;
-const GLib = imports.gi.GLib;
-const GObject = imports.gi.GObject;
-const Meta = imports.gi.Meta;
-const St = imports.gi.St;
-const Pango = imports.gi.Pango;
+const { Atk, Clutter, GLib, GObject, Meta, St, Pango } = imports.gi;
 
 const GrabHelper = imports.ui.grabHelper;
 const Layout = imports.ui.layout;
 const Lightbox = imports.ui.lightbox;
 const Main = imports.ui.main;
-const Tweener = imports.ui.tweener;
 
 const Params = imports.misc.params;
 
@@ -100,20 +92,43 @@ var State = {
 };
 
 
-var BlurEffect = GObject.registerClass(
-class PomodoroBlurEffect extends Clutter.ShaderEffect {
+var BlurEffect = GObject.registerClass({
+    Properties: {
+        'orientation': GObject.ParamSpec.enum(
+            'orientation', 'orientation', 'orientation',
+            GObject.ParamFlags.READWRITE,
+            Clutter.Orientation, Clutter.Orientation.HORIZONTAL
+        ),
+        'brightness': GObject.ParamSpec.float(
+            'brightness', 'brightness', 'brightness',
+            GObject.ParamFlags.READWRITE,
+            0, 1, 1
+        ),
+        'factor': GObject.ParamSpec.float(
+            'factor', 'factor', 'factor',
+            GObject.ParamFlags.READWRITE,
+            0, 1, 0
+        ),
+    },
+}, class PomodoroBlurEffect extends Clutter.ShaderEffect {
     _init(params) {
-        params = Params.parse(params, { orientation: Clutter.Orientation.HORIZONTAL,
-                                        brightness: 1.0,
-                                        factor: 1.0 });
+        params = Params.parse(params, {
+            orientation: Clutter.Orientation.HORIZONTAL,
+            brightness: 1.0,
+            factor: 0.0,
+        });
+
+        this._orientation = undefined;
+        this._brightness = undefined;
+        this._factor = undefined;
 
         super._init({ shader_type: Clutter.ShaderType.FRAGMENT_SHADER });
 
+        this.set_shader_source(BLUR_FRAGMENT_SHADER);
+
         this.orientation = params.orientation;
         this.brightness = params.brightness;
         this.factor = params.factor;
-
-        this.set_shader_source(BLUR_FRAGMENT_SHADER);
     }
 
     get brightness() {
@@ -121,8 +136,11 @@ class PomodoroBlurEffect extends Clutter.ShaderEffect {
     }
 
     set brightness(value) {
-        this.set_uniform_value('brightness', GObject.Float(value));
+        if (this._brightness == value)
+            return;
         this._brightness = value;
+        this.set_uniform_value('brightness', GObject.Float(this._brightness));
+        this.notify('brightness');
     }
 
     get factor() {
@@ -130,12 +148,26 @@ class PomodoroBlurEffect extends Clutter.ShaderEffect {
     }
 
     set factor(value) {
-        this.set_uniform_value('factor', GObject.Float(value));
+        if (this._factor == value)
+            return;
         this._factor = value;
+        this.set_uniform_value('factor', GObject.Float(this._factor));
+        this.notify('factor');
     }
 
-    vfunc_pre_paint() {
-        let res = super.vfunc_pre_paint();
+    get orientation() {
+        return this._orientation;
+    }
+
+    set orientation(value) {
+        if (this._orientation == value)
+            return;
+        this._orientation = value;
+        this.notify('orientation');
+    }
+
+    vfunc_pre_paint(context) {
+        let res = super.vfunc_pre_paint(context);
         let [success, width, height] = this.get_target_size();
 
         if (success) {
@@ -159,107 +191,99 @@ class PomodoroBlurEffect extends Clutter.ShaderEffect {
 });
 
 
-var BlurredLightbox = class extends Lightbox.Lightbox {
-    constructor(container, params) {
+var BlurredLightbox = GObject.registerClass(
+class extends Lightbox.Lightbox {
+    _init(container, params) {
         params.radialEffect = false;
 
-        super(container, params);
+        super._init(container, params);
 
         if (Clutter.feature_available(Clutter.FeatureFlags.SHADERS_GLSL)) {
-            // TODO: Try consolidate these effects into one
-            this._blurXEffect = new BlurEffect({ orientation: Clutter.Orientation.HORIZONTAL,
-                                                 brightness: this._fadeFactor,
-                                                 factor: 0.0 });
-            this._blurYEffect = new BlurEffect({ orientation: Clutter.Orientation.VERTICAL,
-                                                 brightness: this._fadeFactor,
-                                                 factor: 0.0 });
-
-            this._blurXEffect.set_enabled(false);
-            this._blurYEffect.set_enabled(false);
-
             // Clone the group that contains all of UI on the screen.  This is the
             // chrome, the windows, etc.
             let uiGroupClone = new Clutter.Clone({ source: Main.uiGroup,
                                                    clip_to_allocation: true });
-            uiGroupClone.add_effect(this._blurXEffect);
-            uiGroupClone.add_effect(this._blurYEffect);
+            uiGroupClone.add_effect_with_name('blur1',
+                new BlurEffect({
+                    orientation: Clutter.Orientation.HORIZONTAL,
+                    brightness: this._fadeFactor,
+                    factor: 0.0,
+                }));
+            uiGroupClone.add_effect_with_name('blur2',
+                new BlurEffect({
+                    orientation: Clutter.Orientation.VERTICAL,
+                    brightness: this._fadeFactor,
+                    factor: 0.0,
+                }));
 
-            this.actor.add_actor(uiGroupClone);
-            this.actor.opacity = 255;
+            this.set_child(uiGroupClone);
+
+            this._uiGroup = uiGroupClone;
+
+            this.set({ opacity: 255 });
+        }
+        else {
+            this._uiGroup = null;
         }
 
-        this.actor.add_style_class_name('extension-pomodoro-lightbox');
+        this.add_style_class_name('extension-pomodoro-lightbox');
     }
 
-    show(fadeInTime) {
-        fadeInTime = fadeInTime || 0;
+    lightOn(fadeInTime) {
+        this.remove_all_transitions();
 
-        if (this._blurXEffect) {
-            Tweener.removeTweens(this._blurXEffect);
-            Tweener.addTween(this._blurXEffect,
-                             { factor: 1.0,
-                               time: fadeInTime,
-                               transition: 'easeOutQuad',
-                               onUpdate: () => {
-                                   this._blurYEffect.factor = this._blurXEffect.factor;
-                               },
-                               onComplete: () => {
-                                   this.shown = true;
-                                   this.emit('shown');
-                               }
-                             });
-            this._blurXEffect.set_enabled(true);
-            this._blurYEffect.set_enabled(true);
+        let easeProps = {
+            duration: fadeInTime || 0,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+        };
+
+        let onComplete = () => {
+            this._active = true;
+            this.notify('active');
+        };
+
+        this.show();
+
+        if (this._uiGroup) {
+            this._uiGroup.remove_all_transitions();
+
+            this._uiGroup.ease_property(
+                '@effects.blur1.factor', 1.0, easeProps);
+            this._uiGroup.ease_property(
+                '@effects.blur2.factor', 1.0, Object.assign({ onComplete }, easeProps));
         } else {
-            Tweener.removeTweens(this.actor);
-            Tweener.addTween(this.actor,
-                             { opacity: 255 * this._fadeFactor,
-                               time: fadeInTime,
-                               transition: 'easeOutQuad',
-                               onComplete: () => {
-                                   this.shown = true;
-                                   this.emit('shown');
-                               }
-                             });
+            this.ease(Object.assign(easeProps, {
+                opacity: 255 * this._fadeFactor,
+                onComplete,
+            }));
         }
-
-        this.actor.show();
     }
 
-    hide(fadeOutTime) {
-        fadeOutTime = fadeOutTime || 0;
+    lightOff(fadeOutTime) {
+        this.remove_all_transitions();
 
-        this.shown = false;
+        this._active = false;
+        this.notify('active');
 
-        if (this._blurXEffect) {
-            Tweener.removeTweens(this._blurXEffect);
-            Tweener.addTween(this._blurXEffect,
-                             { factor: 0.0,
-                               time: fadeOutTime,
-                               transition: 'easeOutQuad',
-                               onUpdate: () => {
-                                   this._blurYEffect.factor = this._blurXEffect.factor;
-                               },
-                               onComplete: () => {
-                                   this._blurXEffect.set_enabled(false);
-                                   this._blurYEffect.set_enabled(false);
+        let easeProps = {
+            duration: fadeOutTime || 0,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+        };
 
-                                   this.actor.hide();
-                               }
-                             });
+        let onComplete = () => this.hide();
+
+        if (this._uiGroup) {
+            this._uiGroup.remove_all_transitions();
+
+            this._uiGroup.ease_property(
+                '@effects.blur1.factor', 0.0, easeProps);
+            this._uiGroup.ease_property(
+                '@effects.blur2.factor', 0.0, Object.assign({ onComplete }, easeProps));
         } else {
-            Tweener.removeTweens(this.actor);
-            Tweener.addTween(this.actor,
-                             { opacity: 0,
-                               time: fadeOutTime,
-                               transition: 'easeOutQuad',
-                               onComplete: () => {
-                                   this.actor.hide();
-                               }
-                             });
+            this.ease(Object.assign(easeProps, { opacity: 0, onComplete }));
         }
     }
-};
+});
 
 
 /**
@@ -303,10 +327,9 @@ var ModalDialog = class {
                                              { fadeFactor: FADE_IN_OPACITY,
                                                inhibitEvents: false });
         this._lightbox.highlight(this._layout);
-        this._lightbox.actor.show();
 
         this._grabHelper = new GrabHelper.GrabHelper(this.actor);
-        this._grabHelper.addActor(this._lightbox.actor);
+        this._grabHelper.addActor(this._lightbox);
 
         global.stage.add_actor(this.actor);
         global.focus_manager.add_group(this.actor);
@@ -319,26 +342,26 @@ var ModalDialog = class {
     _addMessageTray() {
         let messageTray = Main.messageTray;
 
-        messageTray.actor.ref();
+        messageTray.ref();
 
-        Main.layoutManager.removeChrome(messageTray.actor);
+        Main.layoutManager.removeChrome(messageTray);
 
-        global.stage.add_child(messageTray.actor);
+        global.stage.add_child(messageTray);
 
-        messageTray.actor.unref();
+        messageTray.unref();
         messageTray.bannerBlocked = false;
     }
 
     _removeMessageTray() {
         let messageTray = Main.messageTray;
 
-        messageTray.actor.ref();
+        messageTray.ref();
 
-        global.stage.remove_child(messageTray.actor);
+        global.stage.remove_child(messageTray);
 
-        Main.layoutManager.addChrome(messageTray.actor, { affectsInputRegion: false });
+        Main.layoutManager.addChrome(messageTray, { affectsInputRegion: false });
 
-        messageTray.actor.unref();
+        messageTray.unref();
     }
 
     open(animate) {
@@ -349,7 +372,8 @@ var ModalDialog = class {
         this.state = State.OPENING;
 
         if (this._pushModalDelaySource == 0) {
-            this._pushModalDelaySource = Mainloop.timeout_add(
+            this._pushModalDelaySource = GLib.timeout_add(
+                        GLib.PRIORITY_DEFAULT,
                         Math.max(MIN_DISPLAY_TIME - IDLE_TIME_TO_PUSH_MODAL, 0),
                         this._onPushModalDelayTimeout.bind(this));
         }
@@ -358,30 +382,30 @@ var ModalDialog = class {
         this._monitorConstraint.index = typeof(global.display) === 'object' && typeof(global.display.get_current_monitor) !== 'undefined'
             ? global.display.get_current_monitor() : global.screen.get_current_monitor();
 
-        this.actor.raise_top();
+        global.stage.set_child_above_sibling(this.actor, null);
         this.actor.show();
 
-        Tweener.removeTweens(this.actor);
+        this.actor.remove_all_transitions();
 
         this._addMessageTray();
 
         if (animate) {
-            this._lightbox.show(FADE_IN_TIME / 1000);
-            Tweener.addTween(this.actor,
-                             { opacity: 255,
-                               time: FADE_IN_TIME / 1000,
-                               transition: 'easeOutQuad',
-                               onComplete: () => {
-                                   if (this.state == State.OPENING) {
-                                       this.state = State.OPENED;
-                                       this.emit('opened');
-                                   }
-                               }
-                             });
+            this._lightbox.lightOn(FADE_IN_TIME);
+            this.actor.ease({
+                opacity: 255,
+                duration: FADE_IN_TIME,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                onComplete: () => {
+                    if (this.state == State.OPENING) {
+                        this.state = State.OPENED;
+                        this.emit('opened');
+                    }
+                }
+            });
             this.emit('opening');
         }
         else {
-            this._lightbox.show();
+            this._lightbox.lightOn();
             this.actor.opacity = 255;
 
             this.state = State.OPENED;
@@ -401,31 +425,31 @@ var ModalDialog = class {
         this.state = State.CLOSING;
         this.popModal();
 
-        Tweener.removeTweens(this.actor);
+        this.actor.remove_all_transitions();
 
         if (animate) {
-            this._lightbox.hide(FADE_OUT_TIME / 1000);
-            Tweener.addTween(this.actor,
-                             { opacity: 0,
-                               time: FADE_OUT_TIME / 1000,
-                               transition: 'easeOutQuad',
-                               onComplete: () => {
-                                   if (this.state == State.CLOSING) {
-                                       this.state = State.CLOSED;
-                                       this.actor.hide();
+            this._lightbox.lightOff(FADE_OUT_TIME);
+            this.actor.ease({
+                opacity: 0,
+                duration: FADE_OUT_TIME,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                onComplete: () => {
+                    if (this.state == State.CLOSING) {
+                        this.state = State.CLOSED;
+                        this.actor.hide();
 
-                                       this._removeMessageTray();
+                        this._removeMessageTray();
 
-                                       this.emit('closed');
-                                   }
-                               }
-                             });
+                        this.emit('closed');
+                    }
+                }
+            });
             this.emit('closing');
         }
         else {
             this.actor.opacity = 0;
             this.actor.hide();
-            this._lightbox.hide();
+            this._lightbox.lightOff();
 
             this.state = State.CLOSED;
 
@@ -461,8 +485,8 @@ var ModalDialog = class {
         }
 
         return this._grabHelper.grab({
-            actor: this._lightbox.actor,
-            focus: this._lightbox.actor,
+            actor: this._lightbox,
+            focus: this._lightbox,
             onUngrab: this._onUngrab.bind(this)
         });
     }
@@ -496,12 +520,13 @@ var ModalDialog = class {
 
         this._disconnectSignals();
 
-        this._lightbox.actor.reactive = true;
+        this._lightbox.reactive = true;
 
         // this._grabHelper.ignoreRelease();
 
         /* delay pushModal to ignore current events */
-        Mainloop.idle_add(
+        GLib.idle_add(
+            GLib.PRIORITY_DEFAULT,
             () => {
                 this._pushModalTries = 1;
 
@@ -509,8 +534,9 @@ var ModalDialog = class {
                     /* dialog became modal */
                 }
                 else {
-                    this._pushModalSource = Mainloop.timeout_add(Math.floor(1000 / PUSH_MODAL_RATE),
-                                                                 this._onPushModalTimeout.bind(this));
+                    this._pushModalSource = GLib.timeout_add(GLib.PRIORITY_DEFAULT,
+                                                             Math.floor(1000 / PUSH_MODAL_RATE),
+                                                             this._onPushModalTimeout.bind(this));
                 }
 
                 return GLib.SOURCE_REMOVE;
@@ -524,10 +550,9 @@ var ModalDialog = class {
      */
     popModal() {
         try {
-            if (this._grabHelper.isActorGrabbed(this._lightbox.actor))
-            {
+            if (this._grabHelper.isActorGrabbed(this._lightbox)) {
                 this._grabHelper.ungrab({
-                    actor: this._lightbox.actor
+                    actor: this._lightbox
                 });
             }
         }
@@ -540,12 +565,12 @@ var ModalDialog = class {
 
     _disconnectSignals() {
         if (this._pushModalDelaySource) {
-            Mainloop.source_remove(this._pushModalDelaySource);
+            GLib.source_remove(this._pushModalDelaySource);
             this._pushModalDelaySource = 0;
         }
 
         if (this._pushModalSource) {
-            Mainloop.source_remove(this._pushModalSource);
+            GLib.source_remove(this._pushModalSource);
             this._pushModalSource = 0;
         }
 
@@ -593,29 +618,35 @@ var PomodoroEndDialog = class extends ModalDialog {
         this._eventId                    = 0;
         this._styleChangedId             = 0;
 
-        this._minutesLabel = new St.Label();
-        this._separatorLabel = new St.Label({ text: ":" });
-        this._secondsLabel = new St.Label();
+        this._minutesLabel = new St.Label({
+            x_expand: true,
+            x_align: Clutter.ActorAlign.END,
+        });
+        this._separatorLabel = new St.Label({
+            text: ":",
+        });
+        this._secondsLabel = new St.Label({
+            x_expand: true,
+            x_align: Clutter.ActorAlign.START,
+        });
 
         let hbox = new St.BoxLayout({ vertical: false, style_class: 'extension-pomodoro-dialog-timer' });
-        hbox.add(this._minutesLabel);
-        hbox.add(this._separatorLabel);
-        hbox.add(this._secondsLabel);
+        hbox.add_actor(this._minutesLabel);
+        hbox.add_actor(this._separatorLabel);
+        hbox.add_actor(this._secondsLabel);
 
         this._descriptionLabel = new St.Label({
-                                       style_class: 'extension-pomodoro-dialog-description',
-                                       text: this.description });
+            style_class: 'extension-pomodoro-dialog-description',
+            text: this.description,
+            x_align: Clutter.ActorAlign.CENTER,
+        });
         this._descriptionLabel.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
         this._descriptionLabel.clutter_text.line_wrap = true;
 
         let box = new St.BoxLayout({ style_class: 'extension-pomodoro-dialog-box',
                                      vertical: true });
-        box.add(hbox,
-                { x_fill: false,
-                  x_align: St.Align.MIDDLE });
-        box.add(this._descriptionLabel,
-                { x_fill: false,
-                  x_align: St.Align.MIDDLE });
+        box.add_actor(hbox);
+        box.add_actor(this._descriptionLabel);
         this._layout.add_actor(box);
 
         this._actorMappedId = this.actor.connect('notify::mapped', this._onActorMappedChanged.bind(this));
@@ -674,7 +705,7 @@ var PomodoroEndDialog = class extends ModalDialog {
         this._cancelOpenWhenIdle();
 
         if (this._closeWhenActiveDelaySource) {
-            Mainloop.source_remove(this._closeWhenActiveDelaySource);
+            GLib.source_remove(this._closeWhenActiveDelaySource);
             this._closeWhenActiveDelaySource = 0;
         }
 
@@ -767,7 +798,9 @@ var PomodoroEndDialog = class extends ModalDialog {
 
         /* Wait until user has a chance of seeing the dialog */
         if (this._closeWhenActiveDelaySource == 0) {
-            this._closeWhenActiveDelaySource = Mainloop.timeout_add(MIN_DISPLAY_TIME,
+            this._closeWhenActiveDelaySource = GLib.timeout_add(
+                GLib.PRIORITY_DEFAULT,
+                MIN_DISPLAY_TIME,
                 () => {
                     if (this._idleMonitor.get_idletime() < IDLE_TIME_TO_CLOSE) {
                         /* Wait until user becomes slightly idle */
@@ -821,7 +854,7 @@ var PomodoroEndDialog = class extends ModalDialog {
 
     _cancelCloseWhenActive() {
         if (this._eventId) {
-            this._lightbox.actor.disconnect(this._eventId);
+            this._lightbox.disconnect(this._eventId);
             this._eventId = 0;
         }
     }
@@ -835,7 +868,7 @@ var PomodoroEndDialog = class extends ModalDialog {
         if (this._eventId == 0) {
             this._eventX = -1;
             this._eventY = -1;
-            this._eventId = this._lightbox.actor.connect('event', this._onEvent.bind(this));
+            this._eventId = this._lightbox.connect('event', this._onEvent.bind(this));
         }
     }
 
