@@ -18,29 +18,20 @@
  *          Kamil Prusko <kamilprusko@gmail.com>
  */
 
-const Mainloop = imports.mainloop;
 const Cairo = imports.cairo;
 const Signals = imports.signals;
 
-const Clutter = imports.gi.Clutter;
-const Gio = imports.gi.Gio;
-const GLib = imports.gi.GLib;
-const GObject = imports.gi.GObject;
-const Gtk = imports.gi.Gtk;
-const Meta = imports.gi.Meta;
-const Pango = imports.gi.Pango;
-const Shell = imports.gi.Shell;
-const St = imports.gi.St;
+const { Clutter, Gio, GLib, GObject, Gtk, Meta, Pango, Shell, St } = imports.gi;
 
 const Extension = imports.misc.extensionUtils.getCurrentExtension();
 const Main = imports.ui.main;
 const MessageTray = imports.ui.messageTray;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
-const Tweener = imports.ui.tweener;
 
 const Config = Extension.imports.config;
 const Timer = Extension.imports.timer;
+const Utils = Extension.imports.utils;
 
 const Gettext = imports.gettext.domain(Config.GETTEXT_PACKAGE);
 const _ = Gettext.gettext;
@@ -132,7 +123,7 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
         let startAction = this._createActionButton('media-playback-start-symbolic', _("Start Timer"));
         startAction.add_style_class_name('extension-pomodoro-indicator-menu-action-border');
         startAction.connect('clicked', this._onStartClicked.bind(this));
-        toggleItem.actor.add(startAction);
+        toggleItem.add_child(startAction);
 
         let timerItem = new PopupMenu.PopupMenuItem("",
                                            { style_class: 'extension-pomodoro-indicator-menu-timer',
@@ -149,10 +140,11 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
                                                style_class: 'extension-pomodoro-indicator-menu-timer-label-button' });
         timerLabelButton.child = timerLabel;
         timerLabelButton.connect('clicked', this._onTimerClicked.bind(this));
-        timerItem.actor.add(timerLabelButton, { expand: true });
+        timerItem.add_child(timerLabelButton);
 
-        let hbox = new St.BoxLayout();
-        timerItem.actor.add(hbox);
+        let hbox = new St.BoxLayout({ x_align: Clutter.ActorAlign.END,
+                                      x_expand: true });
+        timerItem.add_child(hbox);
 
         let pauseAction = this._createActionButton('media-playback-pause-symbolic', _("Pause Timer"));
         pauseAction.connect('clicked', this._onPauseClicked.bind(this));
@@ -189,7 +181,7 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
                 this._activateState(name);
             });
 
-        menuItem.actor.add_style_class_name('state-item');
+        menuItem.add_style_class_name('state-item');
 
         this._stateItems[name] = menuItem;
 
@@ -221,8 +213,8 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
             this._isPaused = isPaused;
             this._timerState = timerState;
 
-            this._toggleMenuItem.actor.visible = !isRunning;
-            this._timerMenuItem.actor.visible = isRunning;
+            this._toggleMenuItem.visible = !isRunning;
+            this._timerMenuItem.visible = isRunning;
 
             this._timerLabelButton.reactive = isRunning && !isPaused && timerState != Timer.State.POMODORO;
             this.pauseAction.child.icon_name = isPaused
@@ -235,15 +227,15 @@ var IndicatorMenu = class extends PopupMenu.PopupMenu {
             for (let key in this._stateItems) {
                 let stateItem = this._stateItems[key];
 
-                stateItem.actor.visible = isRunning;
+                stateItem.visible = isRunning;
 
                 if (key == timerState) {
                     stateItem.setOrnament(PopupMenu.Ornament.DOT);
-                    stateItem.actor.add_style_class_name('active');
+                    stateItem.add_style_class_name('active');
                 }
                 else {
                     stateItem.setOrnament(PopupMenu.Ornament.NONE);
-                    stateItem.actor.remove_style_class_name('active');
+                    stateItem.remove_style_class_name('active');
                 }
             }
         }
@@ -384,16 +376,18 @@ var TextIndicator = class {
             this._state = state;
 
             if (state == Timer.State.POMODORO) {
-                Tweener.addTween(this.actor,
-                                 { opacity: FADE_IN_OPACITY * 255,
-                                   time: FADE_IN_TIME / 1000,
-                                   transition: 'easeOutQuad' });
+                this.actor.ease({
+                    opacity: FADE_IN_OPACITY * 255,
+                    duration: FADE_IN_TIME,
+                    mode: Clutter.AnimationMode.EASE_OUT_QUAD
+                });
             }
             else {
-                Tweener.addTween(this.actor,
-                                 { opacity: FADE_OUT_OPACITY * 255,
-                                   time: FADE_OUT_TIME / 1000,
-                                   transition: 'easeOutQuad' });
+               this.actor.ease({
+                    opacity: FADE_OUT_OPACITY * 255,
+                    duration: FADE_OUT_TIME,
+                    mode: Clutter.AnimationMode.EASE_OUT_QUAD
+               });
             }
         }
 
@@ -608,10 +602,11 @@ class PomodoroIndicator extends PanelMenu.Button {
 
         this._arrow = PopupMenu.arrowIcon(St.Side.BOTTOM);
         this._blinking = false;
-        this._blinkTimeoutSource = 0;
+        this._blinkingGroup = new Utils.TransitionGroup();
 
         this._hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
         this._hbox.pack_start = true;
+        this._hbox.set_x_align(Clutter.ActorAlign.CENTER);
         this._hbox.set_y_align(Clutter.ActorAlign.CENTER);
         this._hbox.add_child(this._arrow);
         this.add_child(this._hbox);
@@ -619,20 +614,19 @@ class PomodoroIndicator extends PanelMenu.Button {
         this.setMenu(new IndicatorMenu(this));
         this.setType(type);
 
+        this._blinkingGroup.addActor(this._hbox);
+        this._blinkingGroup.addActor(this.menu.timerLabel);
+        this._blinkingGroup.addActor(this.menu.pauseAction.child);
         this._onBlinked();
 
         this._timerPausedId = this.timer.connect('paused', this._onTimerPaused.bind(this));
         this._timerResumedId = this.timer.connect('resumed', this._onTimerResumed.bind(this));
 
         let destroyId = this.connect('destroy', () => {
-            if (this._blinkTimeoutSource != 0) {
-                Mainloop.source_remove(this._blinkTimeoutSource);
-                this._blinkTimeoutSource = 0;
-            }
-
             this.timer.disconnect(this._timerPausedId);
             this.timer.disconnect(this._timerResumedId);
             this.timer = null;
+            this._blinkingGroup.destroy();
 
             if (this.icon) {
                 this.icon.destroy();
@@ -681,46 +675,38 @@ class PomodoroIndicator extends PanelMenu.Button {
 
     _blink() {
         if (!this._blinking) {
-            this._blinking = true;
-
-            let fadeOutParams = {
-                time: FADE_OUT_TIME / 1000,
-                transition: 'easeInOutQuad',
-                opacity: FADE_OUT_OPACITY * 255
-            };
-            let fadeInParams = {
-                time: FADE_IN_TIME / 1000,
-                transition: 'easeInOutQuad',
-                delay: FADE_OUT_TIME / 1000,
-                opacity: FADE_IN_OPACITY * 255,
-                onComplete: this._onBlinked.bind(this)
-            };
-
-            if (Gtk.Settings.get_default().gtk_enable_animations) {
-                Tweener.addTween(this._hbox, fadeOutParams);
-                Tweener.addTween(this._hbox, fadeInParams);
-                Tweener.addTween(this.menu.timerLabel, fadeOutParams);
-                Tweener.addTween(this.menu.timerLabel, fadeInParams);
-                Tweener.addTween(this.menu.pauseAction.child, fadeOutParams);
-                Tweener.addTween(this.menu.pauseAction.child, fadeInParams);
-            }
-            else if (this._blinkTimeoutSource == 0) {
-                Tweener.addTween(this._hbox, fadeOutParams);
-
-                this._blinkTimeoutSource = Mainloop.timeout_add(FADE_OUT_TIME,
-                    () => {
-                        Tweener.addTween(this._hbox, fadeInParams);
-
-                        this._blinkTimeoutSource = Mainloop.timeout_add(FADE_IN_TIME, () => {
-                            this._blinkTimeoutSource = 0;
-
-                            this._onBlinked ();
-
-                            return GLib.SOURCE_REMOVE;
-                        });
-
-                        return GLib.SOURCE_REMOVE;
+            let fadeIn = () => {
+                if (this._blinking) {
+                    this._blinkingGroup.easeProperty('opacity', FADE_IN_OPACITY * 255, {
+                        duration: 1750,
+                        mode: Clutter.AnimationMode.EASE_IN_OUT_CUBIC,
+                        onComplete: fadeOut,
                     });
+                }
+                else {
+                    this._onBlinked();
+                }
+            };
+            let fadeOut = () => {
+                if (this._blinking) {
+                    this._blinkingGroup.easeProperty('opacity', FADE_OUT_OPACITY * 255, {
+                        duration: 1750,
+                        mode: Clutter.AnimationMode.EASE_IN_OUT_CUBIC,
+                        onComplete: fadeIn,
+                    });
+                }
+                else {
+                    this._onBlinked();
+                }
+            };
+
+            if (St.Settings.get().enable_animations) {
+                this._blinking = true;
+                this._blinkingGroup.easeProperty('opacity', FADE_OUT_OPACITY * 255, {
+                    duration: FADE_OUT_TIME,
+                    mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                    onComplete: fadeIn,
+                });
             }
         }
     }
@@ -731,25 +717,11 @@ class PomodoroIndicator extends PanelMenu.Button {
 
     _onTimerResumed() {
         if (this._blinking) {
-            let fadeInParams = {
-                time: 200 / 1000,
-                transition: 'easeOutQuad',
-                opacity: FADE_IN_OPACITY * 255,
+            this._blinkingGroup.easeProperty('opacity', FADE_IN_OPACITY * 255, {
+                duration: 200,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
                 onComplete: this._onBlinked.bind(this)
-            };
-
-            Tweener.removeTweens(this._hbox);
-            Tweener.removeTweens(this.menu.timerLabel);
-            Tweener.removeTweens(this.menu.pauseAction.child);
-
-            Tweener.addTween(this._hbox, fadeInParams);
-            Tweener.addTween(this.menu.timerLabel, fadeInParams);
-            Tweener.addTween(this.menu.pauseAction.child, fadeInParams);
-
-            if (this._blinkTimeoutSource != 0) {
-                Mainloop.source_remove(this._blinkTimeoutSource);
-                this._blinkTimeoutSource = 0;
-            }
+            });
         }
     }
 });
