@@ -18,7 +18,6 @@
  *
  */
 
-const Lang = imports.lang;
 const Signals = imports.signals;
 
 const Shell = imports.gi.Shell;
@@ -28,6 +27,7 @@ const ExtensionSystem = imports.ui.extensionSystem;
 
 const Extension = imports.misc.extensionUtils.getCurrentExtension();
 
+const ENABLED_EXTENSIONS_KEY = 'enabled-extensions';
 
 const VIDEO_PLAYER_CATEGORIES = [
     ['Player', 'Video'],
@@ -36,10 +36,8 @@ const VIDEO_PLAYER_CATEGORIES = [
 ];
 
 
-var Patch = new Lang.Class({
-    Name: 'PomodoroPatch',
-
-    _init(object, overrides) {
+var Patch = class {
+    constructor(object, overrides) {
         this.object = object;
         this.overrides = overrides;
         this.initial = {};
@@ -52,7 +50,7 @@ var Patch = new Lang.Class({
                 logWarning('Property "%s" for %s is not defined'.format(name, this.object));
             }
         }
-    },
+    }
 
     apply() {
         if (!this.applied) {
@@ -64,7 +62,7 @@ var Patch = new Lang.Class({
 
             this.emit('applied');
         }
-    },
+    }
 
     revert() {
         if (this.applied) {
@@ -76,16 +74,87 @@ var Patch = new Lang.Class({
 
             this.emit('reverted');
         }
-    },
+    }
 
     destroy() {
         this.revert();
         this.disconnectAll();
     }
-});
+};
 Signals.addSignalMethods(Patch.prototype);
 
 
+var TransitionGroup = class {
+
+    /* Helper class to share property transition between multiple actors */
+
+    constructor() {
+        this._actors = [];
+        this._transitions = {};  // by name, containing {'target': ..., 'params': ...}
+        this._transitionStoppedId = 0;
+        this._referenceActor = null;
+    }
+
+    _setReferenceActor(actor) {
+        this._referenceActor = actor;
+    }
+
+    addActor(actor) {
+        let index = this._actors.indexOf(actor);
+
+        if (!actor || index >= 0) {
+            return;
+        }
+
+        this._actors.push(actor);
+
+        if (!this._referenceActor) {
+            this._setReferenceActor(actor);
+        }
+    }
+
+    removeActor(actor) {
+        let index = this._actors.indexOf(actor);
+        if (index >= 0) {
+            this._actors.splice(index, 1);
+
+            for (let name in this._transitions) {
+                actor.remove_transition(name);
+            }
+        }
+
+        if (this._referenceActor === actor) {
+            this._setReferenceActor(this._actors.length > 0 ? this._actors[0] : null);
+        }
+    }
+
+    easeProperty(name, target, params) {
+        let onStopped = params.onStopped;
+        let onComplete = params.onComplete;
+
+        params = Object.assign({
+            onStopped: (isFinished) => {
+                if (onStopped)
+                    onStopped(isFinished);
+
+                if (onComplete && isFinished)
+                    onComplete();
+            }
+        }, params);
+
+        this._actors.forEach((actor) => {
+            actor.ease_property(name, target, params);
+        });
+    }
+
+    destroy() {
+        this._actors.forEach((actor) => {
+            this.removeActor(actor);
+        });
+    }
+}
+
+
 function arrayContains(array1, array2) {
     for (let i = 0; i < array2.length; i++) {
         if (array1.indexOf(array2[i]) < 0) {
@@ -156,12 +225,13 @@ function versionCheck(required) {
     return false;
 }
 
+
 function disableExtension(uuid) {
-    let enabledExtensions = global.settings.get_strv(ExtensionSystem.ENABLED_EXTENSIONS_KEY);
+    let enabledExtensions = global.settings.get_strv(ENABLED_EXTENSIONS_KEY);
     let extensionIndex = enabledExtensions.indexOf(uuid);
 
     if (extensionIndex != -1) {
         enabledExtensions.splice(extensionIndex, 1);
-        global.settings.set_strv(ExtensionSystem.ENABLED_EXTENSIONS_KEY, enabledExtensions);
+        global.settings.set_strv(ENABLED_EXTENSIONS_KEY, enabledExtensions);
     }
 }
